using System.Collections.Generic;
using System.Threading.Tasks;
using GxPress.Auth;
using GxPress.Common.Page;
using GxPress.Repository.Interface;
using GxPress.Repository.Interface.Group;
using GxPress.Request.App.Group;
using GxPress.Request.App.GroupUser;
using GxPress.Request.App.Topic;
using GxPress.Result;
using GxPress.Result.App.Group;
using GxPress.Result.App.GroupUser;
using GxPress.Result.App.Topic;
using GxPress.Result.Web;
using GxPress.Service.Interface.Group;
using GxPress.Service.Interface.GroupCategory;
using GxPress.Service.Interface.Topic;
using Microsoft.AspNetCore.Authorization;
using Microsoft.AspNetCore.Mvc;
using Microsoft.Extensions.Logging;

namespace GxPress.Api.AppControllers
{
    /// <summary>
    /// 小组
    /// </summary>
    [Route("api/app/group")]
    [ApiController]
    [Authorize]
    public class GroupController : ControllerBase
    {
        private readonly ILogger<GroupController> _logger;
        private readonly IGroupRepository _groupRepository;
        private readonly IGroupUserRepository _groupUserRepository;
        private readonly IGroupFolderRepository _groupFolderRepository;
        private readonly IGroupService _groupService;
        private readonly ILoginContext _loginContext;
        private readonly ITopicRepository _topicRepository;
        private readonly ITopicService _topicService;
        private readonly IGroupCategoryRepository groupCategoryRepository;
        private readonly IUserGroupCategoryRepository userGroupCategoryRepository;
        private readonly IGroupCategoryService groupCategoryService;
        public GroupController(ILogger<GroupController> logger, IGroupRepository groupRepository,
            IGroupFolderRepository groupFolderRepository, IGroupUserRepository groupUserRepository,
            ILoginContext loginContext, IGroupService groupService, ITopicRepository topicRepository,
            ITopicService topicService, IGroupCategoryRepository groupCategoryRepository,
            IUserGroupCategoryRepository userGroupCategoryRepository, IGroupCategoryService groupCategoryService)
        {
            _logger = logger;
            _groupFolderRepository = groupFolderRepository;
            _groupRepository = groupRepository;
            _groupUserRepository = groupUserRepository;
            _loginContext = loginContext;
            _groupService = groupService;
            _topicRepository = topicRepository;
            _topicService = topicService;
            this.groupCategoryRepository = groupCategoryRepository;
            this.userGroupCategoryRepository = userGroupCategoryRepository;
            this.groupCategoryService = groupCategoryService;
        }

        /// <summary>
        /// 新建小组
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        [HttpPut("insert")]
        public async Task<GroupDetailResult> Insert(GroupInRequest request)
        {
            request.UserId = _loginContext.AccountId;
            var result = await _groupService.InsertGroup(request);
            //插入小组用户
            if (result.Id > 0 && request.UserIds.Count > 0)
            {
                var groupUserInRequest = new GroupUserInRequest();
                groupUserInRequest.UserId = request.UserId;
                groupUserInRequest.UserIds = request.UserIds;
                groupUserInRequest.SourceType = 2;
                groupUserInRequest.GroupId = result.Id;
                await _groupService.InsertsAsync(groupUserInRequest);
            }
            return result;
        }

        /// <summary>
        ///  添加小组成员
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        [HttpPost("insert-user")]
        public async Task<AppResult<object>> AddGroupUser(GroupUserInRequest request)
        {
            request.UserId = _loginContext.AccountId;
            return await _groupService.InsertsAsync(request);
        }

        /// <summary>
        /// 删除小组成员
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        [HttpPost("delete-user")]
        public async Task<bool> DeleteGroupUser(GroupUserDeRequest request)
        {
            request.UserId = _loginContext.AccountId;
            return await _groupService.DeleteAsync(request);
        }

        /// <summary>
        /// 退出小组
        /// </summary>
        /// <param name="groupId"></param>
        /// <returns></returns>
        [HttpDelete("{groupId}")]
        public async Task<bool> DeleteAsync(int groupId)
        {
            return await _groupService.DeleteAsync(_loginContext.AccountId, groupId);
        }

        /// <summary>
        /// 根据文件夹ID查询小组列表
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        [HttpPost("find")]
        public async Task<PagedList<GroupUserFindResult>> FindGroupByGroupFolderId(GroupSearchRequest request)
        {
            request.UserId = _loginContext.AccountId;
            return await _groupService.FindGroupByGroupFolderIdAsync(request);
        }

        /// <summary>
        /// 获取小组详情
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        [HttpPost("detail")]
        public async Task<GroupDetailResult> GroupDetailAsync(GroupDetailRequest request)
        {
            request.UserId = _loginContext.AccountId;
            return await _groupService.GroupDetailAsync(request);
        }

        /// <summary>
        /// 修改小组
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        [HttpPost("update")]
        public async Task<bool> Update(GroupUpRequest request)
        {
            return await _groupRepository.UpdateAsync(request);
        }
        /// <summary>
        /// 是否添加到通讯录
        /// </summary>
        /// <param name="groupId"></param>
        /// <returns></returns>
        [HttpPut("contacts/{groupId}")]
        public async Task<bool> UpdateContactsAsync(int groupId)
        {
            return await _groupRepository.UpdateContactsAsync(groupId, _loginContext.AccountId);
        }
        /// <summary>
        ///设置权限
        /// 1 超级管理员 2 管理员 3普通用户
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        [HttpPost("update-user")]
        public async Task<bool> UpdateGroupUser(GroupUserUpRequest request)
        {
            return await _groupUserRepository.UpdateAsync(request);
        }

        /// <summary>
        /// 根据GroupId获取话题列表
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        [HttpPost("search")]
        public async Task<PagedList<TopicListPageResult>> FindTopicByGroupId(TopicDetailListRequest request)
        {
            request.UserId = _loginContext.AccountId;
            return await _topicService.FindTopicByGroupIdAsync(request);
        }

        /// <summary>
        /// 加入是否管理员同意
        /// </summary>
        /// <param name="groupId"></param>
        /// <returns></returns>
        [HttpPut("set-admin/{groupId}")]
        public async Task<bool> SetGroupIsAdmin(int groupId)
        {
            return await _groupRepository.SetGroupIsAdminAsync(groupId);
        }
        /// <summary>
        /// 是否禁言
        /// </summary>
        /// <param name="groupId"></param>
        /// <returns></returns>
        [HttpPut("set-words/{groupId}")]
        public async Task<bool> SetGroupIsWords(int groupId)
        {
            return await _groupRepository.SetGroupIsWordsAsync(groupId);
        }

        /// <summary>
        /// 扫描二维码获取小组详情
        /// </summary>
        /// <param name="guId"></param>
        /// <returns></returns>
        [HttpGet("{guId}")]
        public async Task<GroupDetailResult> GroupDetailByGuId(string guId)
        {
            var userId = _loginContext.AccountId;
            return await _groupService.GroupDetailByGuIdAsync(guId, userId);
        }

        /// <summary>
        /// 设置用户是否免打扰
        /// </summary>
        /// <param name="groupId"></param>
        /// <returns></returns>
        [HttpPut("set-disturb/{groupId}")]
        public async Task<bool> SetIsUDisturbAsync(int groupId)
        {
            var userId = _loginContext.AccountId;
            return await _groupUserRepository.SetIsUDisturbAsync(groupId, userId);
        }

        /// <summary>
        /// 搜索小组成员
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        [HttpPost("search-group-user")]
        public async Task<IEnumerable<GroupUserListResult>> SearchGroupUserAsync(TopicDetailListRequest request)
        {
            return await _groupRepository.SearchGroupUserAsync(request);
        }
        /// <summary>
        /// 根据广场类别查询小组
        /// </summary>
        /// <param name="categoryId"></param>
        /// <returns></returns>
        [HttpGet("category-list/{categoryId}")]
        public async Task<IEnumerable<SearchGroupDetailResult>> GetAllAsync(int categoryId)
        {
            return await _groupService.GetAllAsync(categoryId, _loginContext.AccountId);
        }
        /// <summary>
        /// 根据parendId获取小组广场
        /// </summary>
        /// <param name="parentId"></param>
        /// <returns></returns>
        [HttpGet("group-category/{parentId}")]
        [AllowAnonymous]
        public async Task<IEnumerable<GroupCategoryResult>> GetGroupCategoryAsync(int parentId)
        {
            return await groupCategoryService.GetGroupCategoryAsync(parentId, _loginContext.AccountId);
        }
        /// <summary>
        /// 或者最近使用的小组
        /// </summary>
        /// <returns></returns>
        [HttpGet("lately-group")]
        public async Task<IEnumerable<LatelyGroupDetailResult>> GetLatelyGroupDetailResultAsync()
        {
            return await _groupRepository.GetLatelyGroupDetailResultAsync(_loginContext.AccountId);
        }
        /// <summary>
        /// 小组转移
        /// </summary>
        /// <param name="request"></param>
        /// /// <returns></returns>
        [HttpPut("transfer")]
        public async Task<bool> SetGroupTransferAsync(GroupTransferReqeust request)
        {
            request.UserId = _loginContext.AccountId;
            return await _groupService.SetGroupTransferAsync(request);
        }

        /// <summary>
        /// /// 小组列表 搜索专用
        /// </summary>
        /// <returns></returns>
        [HttpGet("list")]
        public async Task<IEnumerable<UserGroupResult>> GetUserGroupListAsync()
        {
            return await _groupRepository.GetUserGroupListAsync(_loginContext.AccountId);
        }
        /// <summary>
        /// 小组广场搜索
        /// </summary>
        /// <param name="keyWord"></param>
        /// <returns></returns>
        [HttpGet("search")]
        public async Task<IEnumerable<SearchGroupDetailResult>> GetSearchGroupAsync([FromQuery] string keyWord)
        {
            return await _groupRepository.GetSearchGroupAsync(keyWord, _loginContext.AccountId);
        }
        /// <summary>
        /// 添加小组广场用户排序
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        [HttpPost("user-group-category")]
        public async Task<bool> InsertAsync(IEnumerable<Entity.tede2.Group.UserGroupCategory> request)
        {
            return await userGroupCategoryRepository.InsertAsync(request, _loginContext.AccountId);
        }
    }
}